	program kinetic_sw	

c	------------------------------------------------------------------
c	Given values of the densities (n_0), of the amount of suprathermal
c	electrons (kappa_e) and of the electrons and protons temperature 
c	(T_0^e and T_0^p) at the exobase r_0,, this program solves 
c	iteratively and simultaneously the quasi-neutrality and the zero 
c	current equations at a fixed radial distance r_max. For that
c	purpose, we use the Newton-Raphson method for nonlinear system of 
c	equations, described in Numerical Recipes. The value of r_max is 
c	iteratively increased until the total potential of the protons
c	after r_max is continuous within a precision fixed either by the 
c	program (default value) or by the user. 
c
c	12/09/03 : Version 1.0 sent to CCMC
c	16/10/03 : Version 1.1 with minor improvements sent to CCMC 
c	31/03/06 : Version 1.2 with minor corrections added when kappa=2.0 
c	        is used
c	09/01/07 : Version 1.21 with slight modifications in the qn_alt 
c			routine (hp_rr_r0 et hp_rr_rm), in the formula
c			giving delta_ve right after r_max is found, and the
c			label 190 (now 210) in the solarwind routine.  Also,
c			a test on fn_qn equal to infinity has been added in
c			the solarwind routine. Finally, the relative precision
c			in the call to rtsafe in the routine solarwind has been
c			decreased from 1.d-6 to 1.d-9 
c	13/01/07 : Version 1.21 with slight modification in the routine
c			funcd : the test on the argument x larger than 10 is now
c			replaced by x larger than 15.  This change ensures that
c			the density of the protons calculated in the qn_alt
c			routine (with the Jockers formalism) is strictly equivalent
c			to the density of the protons calculated in the moments
c			routine.
c	14/01/07 : Version 1.12 with slight modification in the routine
c			solarwind : when r > r_max, the search for gp ensuring
c			the quasi-neutrality starts at gp_max+0.1 instead of gp_max
c			-0.01 before.  As a result, three solutions are found
c			for a few altitudes right above r_max where the correct
c			gp is slightly larger than gp_max.  Therefore, the vector
c			sol becomes a vector with three components.
c	15/01/07 : Version 1.21 with slight modifications in the routine 
c			solarwind : when r > r_max, the search for gp ensuring
c			the quasi-neutrality starts at gp_max+0.5 so that we find
c			all three roots for alt < alt_max.  The test on (k eq 2)
c			becomes now a test on (k eq 3) and the gp_sol is chosen
c			accordingly among the sol(k) solutions.  When r < r_max,
c			add of a "goto 208" in order to avoid the first gp as a
c			solution (when fn_qn is negative for the first gp value)
c			and change of gp_max into nint(100*xx(1))/100 in order to
c			find roots located between xx(1) and xx(1)-0.01.
c	17/01/07 : Version 1.21 with slight modification in the routine 
c			solarwind : when r > r_max, the criterium to search for
c			roots of quasi-neutrality has been modified in order to
c			not take into account a root found when fn_qn_old is set
c			to 1.d31 (inf).
c	17/01/07 : Version 1.22 with add of a routine trick_rmax to find the "best"
c			gp solution in the case of a single gp solution between
c			r_max and alt_crit (normally there should be 3 Gp solutions).
c	18/01/07 : Version 1.22 with slight modification in the routine
c			solarwind : the loop on alt is replaced by a loop on an
c			integer between 1 and nbiter and the alt is calculated
c			inside the loop. This allows to actually calculate the
c			electric potential in r_end.
c	19/01/07 : Version 1.22 with slight modification in the routine trick_rmax
c			with the addition of a common/consts so that the routine
c			knows which value inf is referring to.
c
c 	------------------------------------------------------------------

	implicit real*8(a-h,o-z)
	parameter (jmax=20, nbiter=200)
	dimension xx(2),gp_res(2000,4),gp_sol(nbiter)
	dimension elec_pot(nbiter), alt(nbiter), results(5)
	real*8 inf,m_sun,mp,me,L	
	character*1 verif1
	COMMON/consts/inf,bk,ec,m_sun,mp,me,g,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp			

	open(20,file='input.dat', status='unknown')
	open(40,file='idl.dat', status='unknown')
	open(50,file='gp_res.dat', status='unknown')
	open(60,file='moments1.dat', status='unknown')
	open(70,file='moments2.dat', status='unknown')	
c	open(100,file='temp_gp.dat', status='unknown')
	
c	--------------------------------------------------------------------
c	Reading of the input.dat file whose first line must contain in the 
c	right order the radial distance of the exobase (in solar radii), the 
c	temperature of the electrons at the exobase (in K), the temperature 
c	of the protons at the exobase (in K) and the kappa index of the 
c	electrons VDF at the exobase (unitless).
c	--------------------------------------------------------------------	
	
	read(20,*) r_exo, T0e, T0p, pk

c	--------------------------------------------------------------------
c	Test of the compatibility of the r_exo value.
c	r_exo must be chosen between 1.1 and 5 Rs
c	--------------------------------------------------------------------

	if (r_exo .lt. 1.1d0 .or. r_exo .gt. 5.0d0) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'The radial distance of the exobase must be chosen'
	write(6,*) 'between 1.1 and 5.0 solar radii.  Please enter another'
	write(6,*) 'value for r_exo in the input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop
	endif
	
c	--------------------------------------------------------------------
c	Test of the compatibility of the T0e value.
c	T0e must be chosen between 0.8*10e6 K and 2.0*10e6 K
c	--------------------------------------------------------------------	
	
	if (T0e .lt. 0.8d6 .or. T0e .gt. 2.0d6) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'The electron temperature at the exobase must be'
	write(6,*) 'chosen between 0.8*10e6 K and 2.0*10e6 K.  Please'
	write(6,*) 'enter another value of T0e in the input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop 
	endif
	
c	--------------------------------------------------------------------
c	Test of the compatibility of the T0 value.
c	T0p must be chosen between 1.0*10e6 K and 2.0*10e6 K
c	--------------------------------------------------------------------	
	
	if (T0p .lt. 1.0d6 .or. T0p .gt. 2.0d6) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'The proton temperature at the exobase must be'
	write(6,*) 'chosen between 1.0*10e6 K and 2.0*10e6 K. Please enter'
	write(6,*) 'another value of T0p in the input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop 
	endif	
	
c	--------------------------------------------------------------------
c	Test of the compatibility of the pk value.
c	pk must be chosen between 2.0 and 1000.0
c	--------------------------------------------------------------------	
	
	if (pk .lt. 2.0d0 .or. pk .gt. 1000.d0) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'The kappa index of the electrons VDF must be chosen'
	write(6,*) 'between 2.0 and 1000 (Maxwellian case). Please enter'
	write(6,*) 'another value of pk in the input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop 
	endif
	
	if (pk .eq. 2.0d0) then
	print *
	write(6,*) '**************** WARNING !! **************************'
	write(6,*) 'For a value of kappa equal to 2.0, the heat flux of the'
	write(6,*) 'electrons is not defined anymore.  Other lower moments'
	write(6,*) 'are still defined.'
	write(6,*) '*******************************************************'
	print *
	endif

	read(20,*) rm_inf,rm_sup
	
c	--------------------------------------------------------------------
c	rm_inf and rm_sup are the limits of the variation range of r_max.
c	--------------------------------------------------------------------

	if (rm_inf .le. r_exo) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'r_max cannot be lower than r_exo ! Please enter'
	write(6,*) 'another value of rm_inf in the input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop
	endif
	
	if (rm_sup .le. rm_inf) then
	print *
	write(6,*) '**************** WARNING !! ************************'
	write(6,*) 'The domain of possible values of r_max that you give'
	write(6,*) 'is not possible : rm_sup is larger than rm_inf !'
	write(6,*) 'Please enter another value of rm_sup in the '
	write(6,*) 'input.dat file.'
	write(6,*) '****************************************************'
	print *
	stop
	endif
	
	read(20,*) r_end


	print *
	print *, '         ******************************************     '
	print *, '         *                                        *     '
	print *, '         *        WELCOME TO KINETIC_SW           *     '
	print *, '         *                                        *     '
	print *, '         *    AN EXOSPHERIC KINETIC MODEL OF      *     '
	print *, '         *    THE SOLAR WIND WITH A MAXIMUM OF    *     '
	print *, '         *    THE POTENTIAL FOR THE PROTONS       *     '
	print *, '         *                                        *     '
	print *, '         *        VERSION 1.22 JANUARY 2007       *     '
	print *, '         *                                        *     '
	print *, '         *            DEVELOPED BY                *     '
	print *, '         *                                        *     '
	print *, '         *             HERVE LAMY                 *     '
	print *, '         *                                        *     '
	print *, '         *                AND                     *     '
	print *, '         *                                        *     '	
	print *, '         *           VIVIANE PIERRARD             *     '
	print *, '         *                                        *     '
	print *, '         *    INSTITUT D''AERONOMIE SPATIALE DE    *     '
	print *, '         *    BELGIQUE, 3 AVENUE CIRCULAIRE,      *     '
	print *, '         *    1180 BRUXELLES, BELGIUM             *     '
	print *, '         *                                        *     '
	print *, '         ******************************************     '
	
	print *
	print *
	write(6,99) 
	print *
	write(6,100) r_exo
	write(6,101) T0e
	write(6,102) T0p
	write(6,103) pk
	write(6,104) r_end
	print *
     	print *	

99	format(T12,'The input parameters you have requested are ')	
100	format(3X,'- the radial distance of the exobase (in solar radii) ',
     ^	T58,':',T63,F4.2)
101	format(3X,'- the electrons temperature at the exobase (in K)',
     ^	T58,':',T63,D9.3)
102	format(3X,'- the protons temperature at the exobase (in K)',
     ^	T58,':',T63,D9.3)
103 	format(3X,'- the kappa index of the electrons VDF at the exobase ', 
     ^	T58,':',T63,F6.2)
104     format(3X,'- the last radial distance', T58, ':',T63,D9.3)     
105	format(T12,'To change one of these parameters, please edit')    
106	format(T12,'the input.dat file ')
     
	write(6,105) 
	write(6,106)
        print *
     	print *	
	
c	write(6,*) 'Do you want to go on with these parameters ? (Y,N) '
c	read(5,*) verif1
c	
c	if (verif1 .eq. 'N' .or. verif1 .eq. 'n') stop
	
	
c	-------------------------------------------------------------------
c	Start test over r_max values.  The initial search interval for
c	r_max is [rm_inf,rm_sup].  For each value of r_max, the subroutine
c	dicho_rmax returns an integer ind_rmax which is -1 if the value
c	taken for r_max is too low or +1 if is too large.  A first test
c	is made in order to check that the r_max value belongs to the
c	search interval considered.  If not, then a message is sent to the
c	screen asking the user to change the domain of r_max values
c	in the input data file.
c	-------------------------------------------------------------------

	lim=0

	r_max=rm_inf
	print *
	write(6,*) 'The value of r_max considered is : ', rm_inf
	print *
	call dicho_rmax(r_max,lim,ind_rmax,xx,alt_max,gp_res)
	
c	------------------------------------------------------------------
c	Test to check if r_max is larger than rm_inf.  If r_max is lower
c	than rm_inf, the program is stopped and the user has to modify
c	the rm_inf value in the input.dat file.	
c	------------------------------------------------------------------
	if (ind_rmax .eq. 1) then
	write(6,*) 'The correct value of r_max is lower than', rm_inf
	write(6,*) 'Please modify the input.dat file accordingly.'
	stop
	else
	continue
	endif
	
	lim=2
	r_max=rm_sup
	print *
	write(6,*) 'The value of r_max considered is : ', rm_sup
	print *
	call dicho_rmax(r_max,lim,ind_rmax,xx,alt_max,gp_res)
	
c	------------------------------------------------------------------
c	Test to check if r_max is lower than rm_sup.  If r_max is larger
c	than rm_sup, the program is stopped and the user has to modify
c	the rm_sup value in the input.dat file.	
c	------------------------------------------------------------------		
	if (ind_rmax .eq. -1) then
	write(6,*) 'The correct value of r_max is larger than', rm_sup
	write(6,*) 'Please modify the input.dat file accordingly.'
	stop
	else
	continue
	endif
	
c	-----------------------------------------------------------------	
c	Start dichotomy on r_max values	
c
c	If after jmax iterations, r_max was not found with enough precision 
c	(i.e. no ind_rmax=0), the search is stopped and the user must
c	decide by eye inspection if the r_max value found at the end of
c 	the iterations is sufficiently precise.
c	-----------------------------------------------------------------

	lim=2
	do j=1,jmax
	r_max=(rm_inf+rm_sup)/2.d0
	print *
	write(6,*) 'The value of r_max considered is : ', r_max	
	print *
	call dicho_rmax(r_max,lim,ind_rmax,xx,alt_max,gp_res)
	lim=1
	select case (ind_rmax)
	case (-1)
	rm_inf=r_max
	case (1)
	rm_sup=r_max
	case (0)
	goto 107
	case default
	write(6,*) 'There is an error in the evaluation of ind_rmax'
	stop
	end select
	enddo
	print *
	write(6,*) 'The r_max value was not found with enough precision.'
	write(6,*) 'Please have a look at the solutions to check if the'
	write(6,*) 'r_max value is sufficiently close to the correct'
	write(6,*) 'solution.'
	print *
	
c	---------------------------------------------------------------	
c	Calculation of the values of the electric potential at r_exo
c	(delta_ve) and at r_max (delta_ve+delta_vp)
c	---------------------------------------------------------------	
	
107	L=(m_sun*mp*G)/(r_exo*r_sun*bk*t0p)	
	delta_vp=(xx(1)-L*(1.d0-(r_exo/r_max)))/(ec/(bk*T0p))
	delta_ve=(xx(2)*((pk-1.5d0)/pk)-(me*T0p/(mp*T0e))*L)*(bk*T0e/ec)
c	delta_ve=delta_ve*(pk-1.5d0)/pk	
	
	print *
	write(6,110) delta_ve
	print *
	write(6,111) delta_ve+delta_vp
	print *
	
110	format(3X,'The electrostatic potential at the exobase is ',T50,
     ^	': ',F9.4)
111	format(3X,'The electrostatic potential at r_max is ',T50, ': ',
     ^	F9.4) 	

c	write(6,*) 'temp', xx
     
c	--------------------------------------------------------------     
c	We write the three sets of gp solutions in a file named 
c	gp_res.dat, which will be used by the IDL procedure 'rmax.pro' 
c	to check that the r_max value found by the program is correct.
c
c	We also search the radial distance alt_crit where the physical
c	solution 'switches' from the mathematical solution 2 to the
c	mathematical solution 1.  It is found as the last radial 
c	distance where the solution 2 is decreasing with increasing
c	altitude.  The value of alt_crit, together with those of r_max
c	and alt_max are written in the idl.dat file, which will also 
c	be used by the 'rmax.pro' IDL procedure.
c	--------------------------------------------------------------
	
	imax=(alt_max-r_max)*100-5
	alt_crit=r_max
	do i=1,imax
	write(50,112) gp_res(i,1),gp_res(i,2),gp_res(i,3),gp_res(i,4)
	if (gp_res(i+1,3) .lt. gp_res(i,3) .and. gp_res(i+1,3)
     ^	.ne. -999.d0) then 
      	alt_crit=gp_res(i+1,1)
	else
	continue
	endif
	enddo
	
112     format(F8.4,T11,F9.4,T22,F9.4,T33,F9.4) 	
	
     	write(40,*) r_max,alt_max,alt_crit	

c	-------------------------------------------------------------------     
c	We call the subroutine SOLARWIND in order to evaluate the total
c	normalized potential of the protons at every radial distance
c	chosen between r_exo and r_end.  By default, the program considers
c	200 radial distances. If you want to change this, you must change  
c	the nbiter parameter at the beginning of the MAIN program and of 
c	the SOLARWIND and MOMENTS subroutines.
c	-------------------------------------------------------------------

	call solarwind(xx,r_max,alt_crit,gp_sol)

	do i=1,nbiter
	alt(i)=r_exo+(i/dfloat(nbiter))*(r_end-r_exo)
	elec_pot(i)=delta_ve+(gp_sol(i)-L*(1.d0-(r_exo/alt(i))))*
     ^	((bk*T0p)/ec)
	
	do ident=1,2
	call moments(ident,alt(i),r_max,delta_ve,delta_ve+delta_vp,
     ^	elec_pot(i),xx,results)

	if (ident .eq. 1) then
	den_e=results(1)
	flux_e=results(2)
        vbulk_e=flux_e/den_e
     	tepar=(1.d0/bk)*((results(3)/results(1))-me*(vbulk_e**2))
	teperp=(1.d0/bk)*(results(4)/results(1))
	anie=tepar/teperp
	te=(tepar+2.d0*teperp)/3.d0
	heatflux_e=results(5)
	else	
	den_p=results(1)
	flux_p=results(2)
	vbulk_p=flux_p/den_p
     	tppar=(1.d0/bk)*((results(3)/results(1))-mp*(vbulk_p**2))
	tpperp=(1.d0/bk)*(results(4)/results(1))
	anip=tppar/tpperp
	tp=(tppar+2.d0*tpperp)/3.d0
	heatflux_p=results(5)
	endif
	
	enddo
c	end of ident loop

	write(60,113) alt(i),gp_sol(i),elec_pot(i),den_e/1.d6,den_p/1.d6,
     ^  flux_e/1.d12,flux_p/1.d12,vbulk_e/1.d3,vbulk_p/1.d3
     	write(70,114) alt(i),tepar,teperp,te,anie,tppar,tpperp,tp,anip,
     ^  heatflux_e,heatflux_p	
		
	enddo
c	end of i loop	

	
113 	format(F8.4,T10,F8.4,T20,F8.2,T31,F10.4,T44,F10.4,T57,F9.4,T69,
     ^	F9.4,T80,F7.2,T91,F7.2)
114	format(F8.4,T11,F11.2,T25,F11.2,T38,F11.2,T51,F7.3,T60,F11.2,T73,
     ^	F11.2,T86,F11.2,T100,F10.3,T112,F11.8,T125,F10.8) 
		
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
c	----------------------------------------------------------------
c	This block data contains the fundamental constants.
c	----------------------------------------------------------------
	
	BLOCK DATA constants
	real*8 inf,bk,ec,m_sun,mp,me,G,r_sun
	COMMON/consts/inf,bk,ec,m_sun,mp,me,G,r_sun
	data inf/1.d+31/
	data bk/1.38d-23/
        data ec/1.602d-19/
	data m_sun/1.99d30/	
	data mp/1.67d-27/
	data me/9.1207d-31/
	data G/6.6726d-11/
	data r_sun/6.96d8/ 

	end 	
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	

	subroutine dicho_rmax(r_max,lim,ind_rmax,xx,alt_max,gp_res)
	implicit real*8(a-h,o-z)
	parameter(nn=2,ntrial=10,tol1=1.d-3,tol2=1.d-3)	
	dimension xx(nn),sol(3),gp_res(2000,4)	
	real*8 inf,m_sun,mp,me	
	COMMON/consts/inf,bk,ec,m_sun,mp,me,g,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp
	integer dich
	
c	--------------------------------------------------------------------	
c	The nn parameter is the number of non-linear equations to be solved.
c	In the case of a solar wind made of protons and electrons, nn=2.
c	The ntrial parameter is the maximal number of steps in the Newton-
c	Raphson method to converge towards a solution.
c	--------------------------------------------------------------------

c	----------------------------------------------------------------	
c	Initial guess for the solutions of quasi-neutrality and zero
c	current equations.
c	----------------------------------------------------------------			

	xx(1)=0.5d0
	xx(2)=5.0d0

c	----------------------------------------------------------------
c	For each r_max, the subroutine mnewt returns the values of the
c	normalized total potentiel of electrons and protons at r_max.
c	These results are obtained by solving simultaneously two non-
c	linear equations with a Newton-Raphson method.  The results are
c	stored in xx(2).
c	----------------------------------------------------------------		
	
	call mnewt(ntrial,xx,nn,tol1,tol2,r_max)	

c	-----------------------------------------------------------------	
c	The following parameters describe the range of radial distances
c	above r_max where all the mathematical solutions of the global
c	normalized potential of protons will be searched for.  alt_min
c	is slightly located above r_max.  alt_max is initially set to
c	10 solar radii and after is chosen closer to the last radial 
c	distance where there are three possible solutions for the total
c	potential of protons.  The step in radial distance is step_alt 
c	and is set to 0.01 solar radius.
c	-----------------------------------------------------------------
	
	start_alt=0.05d0
	step_alt=0.01d0
	alt_min=(nint(100.d0*r_max)/100.d0)+start_alt
	if (lim .eq. 0) alt_max=10.0d0	

c	----------------------------------------------------------------	
c	Start loop over radial distances for fixed values of r_max, 
c	gp_max and ge_max given by the subroutine mnewt.
c
c	i is an integer used to write the solutions of the quasi-
c	neutrality equation in the gp_res table.
c	----------------------------------------------------------------	
	
180	i=1
	do alt=alt_min,alt_max,step_alt
	
c	---------------------------------------------------------------	
c	k is an integer initially set to 0 and which is increased each
c	time a root of the quasi-neutrality equation is found.
c	k is different for each altitude but has only two possible 
c	values : 1 or 3.
c	---------------------------------------------------------------	

	k=0	
	
c	----------------------------------------------------------------	
c	The following parameters are the range of normalized protons
c	potential gp where roots of the quasi-neutrality equation  
c	den_p-den_e are searched for.  They are expressed in eV units.
c	---------------------------------------------------------------	
	
	gp_min=-5.d0
	gp_max=10.d0
	step_gp=0.01d0	

	gp_old=gp_min
	fn_qn_old=1000.d0
	
c	---------------------------------------------------------------	
c	Start loop over gp.  First, a simple dichotomy is used in order
c	to locate approximately the different roots.  Once their 
c	approximate location is known, a Newton-Raphson method is used 
c	to converge more precisely to the root. The integer 'dich' is
c	set equal to 1 when dichotomy is used and equal to 0 otherwise.
c	When dichotomy is used, the derivative of the quasi-neutrality
c	equation is not calculated in the qn_alt subroutine.
c	---------------------------------------------------------------

	jmin=dint(gp_min/step_gp)
	jmax=dint(gp_max/step_gp)
	
	do j=jmin,jmax,1
	j2=dfloat(j2)
	gp=j/100.d0
	
	dich=1
	call qn_alt(gp,xx,alt,r_max,dich,fn_qn,dfn_qn)	
	
	if (fn_qn .eq. inf) then
	goto 190
	endif
	
	if ((fn_qn*fn_qn_old) .lt. 0.d0) then 
	gp_inf1=gp_old
	gp_sup1=gp
	if (gp_inf1 .eq. gp_sup1) then
	k=k+1
	sol(k)=gp_min
	else
	k=k+1
	dich=0
	sol(k)=rtsafe(xx,alt,r_max,dich,gp_inf1,gp_sup1,1.d-6)
	endif
	else
	endif	
	
	fn_qn_old=fn_qn
	gp_old=gp	
	
	enddo
c	end of gp loop
	
	
c	----------------------------------------------------------------	
c	All the gp solutions are written in the gp_res matrix
c	together with the radial distance.  Depending on the value 
c	of k, there is one or three solutions for gp.  If k=1, then   
c	the two other roots are set equal to -999 in this matrix. 
c	----------------------------------------------------------------	
	
190	if (k .eq. 3) then
	gp_res(i,1)=alt
	gp_res(i,2)=sol(1)
	gp_res(i,3)=sol(2)
	gp_res(i,4)=sol(3)
	else
	if(sol(k) .lt. xx(1)) then
	gp_res(i,1)=alt
	gp_res(i,2)=sol(1)
	gp_res(i,3)=-999.d0
	gp_res(i,4)=-999.d0
	continue
	else
	gp_res(i,1)=alt
	gp_res(i,2)=-999.d0
	gp_res(i,3)=-999.d0
	gp_res(i,4)=sol(1)
	endif
	endif	
			
	i=i+1
	
	enddo	
c	End of alt loop	


c	----------------------------------------------------------------
c	For the two first values of r_max, the program tests if
c	altmax is sufficiently large to cover the whole area where
c	gp has three solutions. If yes, then the program simply
c	continues.  If no, we start the calculations again with a 
c	value of altmax incremented by two solar radii.
c	---------------------------------------------------------------- 

	if (lim .eq. 0) then
	
	if (gp_res(i-1,2) .ne. -999.d0 .and. gp_res(i-1,3) .ne. -999d0
     ^	.and. gp_res(i-1,4) .ne. -999d0) then
	alt_max=alt_max+2.0d0
	print *
	write(6,*) 'The value of alt_max is too small.  We perform a new'
	write(6,*) 'test with alt_max = ', alt_max
	print *
	goto 180
	else
	if (gp_res(i-1,4) .ge. xx(1)) then
	alt_max=alt_max+2.0d0
	print *
	write(6,*) 'The value of r_max AND the value of alt_max are too'
	write(6,*) 'small. We perform a new test with alt_max = ', alt_max
	print *	
	goto 180
	else
	continue
	endif 
	endif
	
	endif	

c	----------------------------------------------------------------	
c	After the check that r_max belongs to the interval chosen by 
c	the user, we determine the maximal radial distance to which
c	solutions of gp will be searched for the next values of r_max.
c	For that purpose, the table gp_res is read from the end and 
c	the new value of alt_max is chosen as the radial distance 
c 	where there are three solutions for the first time,  
c	incremented by two solar radii.  This test is only made once 
c	by testing if the integer 'lim' is equal to 2).
c	-----------------------------------------------------------------

	if (lim .eq. 2) then
	jmax=(alt_max-alt_min)/step_alt
c	jmax is the number of radial distance considered	
	do j=jmax,1,-1
		if (gp_res(j,2) .ne. -999.d0 .and. gp_res(j,3) .ne. -999.d0 
     ^		.and. gp_res(j,4) .ne. -999.d0) then
		alt_max=gp_res(j,1)+1.d0
		goto 200
		else
		continue
		endif
	enddo	
	endif

c	----------------------------------------------------------------
c	Test to check whether the r_max value considered is smaller
c	or larger than the "true" r_max ensuring continuity of the
c	gp solution for all altitudes above r_max. 
c 
c	We start from the altitude = r_max + 0.05 Rs in order to avoid
c	a very small region just after r_max where only one solution
c	is found (see paper by Jockers [1970] for more details).
c
c	We are first looking at -999 values for the lower solution of
c	g_p by reading the second column of the gp_res matrix.  If two
c	-999 values are found consecutively, it means that the value 
c	considered for r_max is too small.
c	----------------------------------------------------------------
	
200	jmin=5
	jmax=(alt_max-alt_min)/step_alt
	sol2_min=-1000.d0
	do j=jmin,jmax-1
	if (gp_res(j,2) .eq. -999.d0) then

c	----------------------------------------------------------------
c	First -999 value detected for the lower gp solution.
c	We test if gp at the next altitude is also equal to -999.
c	----------------------------------------------------------------

		if (gp_res(j+1,2) .eq. -999.d0) then

c	---------------------------------------------------------------		
c	If yes, the r_max value considered is smaller than the 'real'
c	r_max.  The integer ind_rmax is -1 and we can return to the
c	main program to test the next value of r_max.
c	---------------------------------------------------------------
				
		ind_rmax=-1
		return	
			
		else
c	--------------------------------------------------------------		
c	If we have only one radial distance with one solution for gp,
c	then we are close enough to the "real" rmax. The integer 
c	ind_rmax is 0 and we can return to the main program and stop
c	the search for r_max.
c	--------------------------------------------------------------
		
		ind_rmax=0		
		
		endif
	
	else	

c	--------------------------------------------------------------	
c	We do not detect -999 value for the lower gp_sol.  Now we
c	have to compare the minimum value of the second gp solution 
c	with the maximum value of the lower gp in order to check if
c	the value of r_max considered is larger than the 'real' r_max.
c	--------------------------------------------------------------
	
		if (gp_res(j+1,2) .gt. gp_res(j,2)) then
		sol1_max=gp_res(j+1,2)
		else
		endif	
		if (gp_res(j+1,3) .lt. gp_res(j,3) .and. gp_res(j+1,3)
     ^		.ne. -999.d0) then
		sol2_min=gp_res(j+1,3)
		else
		endif

	endif			
	enddo
c	End j loop

	if (sol2_min .eq. -1000.d0) then
	ind_rmax=1
	return
	endif


c	---------------------------------------------------------------
c	If the difference between the minimum value of the second gp
c	solution and the maximum value of the lower gp solution is
c	lower than a given tolerance (here 0.02d0), then we are close
c	enough to the real r_max.  The integer ind_rmax is 0 and we
c	can return to the main program and stop the search for r_max.
c	Otherwise, the integer ind_rmax is 1 and we return to the main
c	program to test the next value of r_max
c	---------------------------------------------------------------

	if (dabs(sol2_min-sol1_max) .le. 0.02d0) then
	ind_rmax=0	
	else
	ind_rmax=1	
	continue
	endif
	
	return
	end
		
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	


	subroutine mnewt(ntrial,x,n,tolx,tolf,r_max)
	integer n,ntrial,np
	real*8 tolf,tolx,x(n),r_max
	parameter (np=2)
	integer i,k,indx(np)
	real*8 d,errf,errx,fjac(np,np),fvec(np),p(np)
		
	do k=1,ntrial
	call usrfun(x,n,np,r_max,fvec,fjac)
	errf=0.
	do i=1,n
	errf=errf+dabs(fvec(i))
	enddo 
	if (errf.le.tolf) return
	do i=1,n
	p(i)=-fvec(i)
	enddo
	call ludcmp(fjac,n,np,indx,d)
	call lubksb(fjac,n,np,indx,p)
	errx=0.
	do i=1,n
	errx=errx+abs(p(i))
	x(i)=x(i)+p(i)
	enddo
	if (errx .le. tolx) return
	enddo
	return
	end
	
	
	
c	****************************************************************
c	****************************************************************
c	****************************************************************


	subroutine usrfun(x,n,np,r_max,fvec,fjac)
	implicit real*8(A-H,O-Z)
	dimension x(n),fvec(n),fjac(n,n)
	
	call funcv(n,x,r_max,fvec)
	call fdjac(n,x,r_max,fvec,np,fjac)
	
	return
	end
		
c	****************************************************************
c	****************************************************************
c	****************************************************************



c	-----------------------------------------------------------------
c	This subroutine evaluates the quasi-neutrality equation (den_p-
c	den_e) and the zero-current equation (flux_p-flux_e) at r=r_max 
c	for given values of the two-dimensional input vector x. x(1) is 
c	the normalized total potential of protons and x(2) is the 
c	normalized total potential of electrons.
c	-----------------------------------------------------------------	
	
	subroutine funcv(n,x,r_max,fn)
        implicit real*8(A-H,O-Z)
	dimension x(n),fn(n)
	real*8 inf,m_sun,mp,me,L,k2_vinf
	COMMON/consts/inf,bk,ec,m_sun,mp,me,G,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp	

	pi=4.d0*datan(1.d0)
	L=(m_sun*mp*G)/(r_exo*r_sun*bk*T0p)
	eta_m=(r_exo/r_max)**2	
	hp=x(1)/(1.d0-eta_m)

c	-----------------------------------------------------------------	
c	Evaluation of ge_rm, the normalized total potential of electrons  
c	at r=r_max 
c	-----------------------------------------------------------------	
	
	ge_rm=(L*(1-(r_exo/r_max))*(1+(me/mp))-x(1))*(T0p/T0e)
	ge_rm=ge_rm*(pk/(pk-1.5d0))
	
c	-----------------------------------------------------------------
c	Evaluation of several quantities related to the calculation
c	of den_e at r=r_max (see Pierrard & Lemaire JGR 101, 7923, 1996
c	for details). fnorme is a normalization factor introduced in 
c	order to recover n_0 for the electrons at the exobase.  fnorme 
c	< 1 because no incoming particles are considered in this model.
c	-----------------------------------------------------------------

	b=1.d0/(1.d0+(x(2)/pk))     
     	beta2b=zzk(pk-0.5d0,1.5d0,b)
	fnorme=1.d0-0.5d0*beta2b		

	ak=dexp(gammln(pk+1.d0)-gammln(pk-0.5d0)-gammln(1.5d0))		
	d=1.d0+(ge_rm/pk)
	b=1.d0/(1.d0+((x(2)-ge_rm)/(pk+ge_rm)))
	alpha=dsqrt(1.d0-eta_m)*(1.d0+(eta_m/(1.d0-eta_m))*
     ^	(ge_rm/(pk+ge_rm)))**(-pk-1.d0)
	c=1.d0+(ge_rm/((1.d0-eta_m)*pk))
	f=1.d0/(1.d0+((1.d0-eta_m)*x(2)-ge_rm)/((1.d0-eta_m)*
     ^	pk+ge_rm))
     
     	if (b .gt. 1.d0) then 
	b=1.d0
	elseif (b .lt. 0.d0) then
	b=0.d0
	endif
	if (f .gt. 1.d0) then 
	f=1.d0
	elseif (f .lt. 0.d0) then
	f=0.d0
	endif
	
	beta2b=zzk(pk-0.5d0,1.5d0,b)
	beta2f=zzk(pk-0.5d0,1.5d0,f)
	
c	-----------------------------------------------------------------
c	Evaluation of several quantities related to the calculation
c	of den_p at r=r_max (see Lamy et al. JGR 108, A1, 1047, 2003
c	for details). fnormp is a normalization factor introduced in 
c	order to recover n_0 for the protons at the exobase.  fnormp 
c	< 1 because no incoming particles are considered in this model.
c	The K2 and W2 functions are evaluated by means of the erfcc and
c	dawson functions because the subroutines used to determine 
c	these latter functions are more precise when their arguments
c	are large (>~ 4)
c	-----------------------------------------------------------------
		
	fac_b=dsqrt((1.d0-eta_m)/eta_m)*dexp(-x(1)/(1.d0-eta_m))
	if (x(1) .lt. 0.d0) then
	print *
	write(6,*) 'Newton-Raphson cannot converge.'
	print *
	write(6,*) 'Your lower limit of r_max is too close to r_exo !'
	write(6,*) 'Please enter another range of r_max values.'
	print *
	stop
	else
	vinf=dsqrt(x(1))
	endif
	k2_vinf=0.5d0*(1.d0-erfcc(vinf))-vinf*dexp(-(vinf**2))/dsqrt(pi)
	vsig=dsqrt(eta_m*x(1)/(1.d0-eta_m))
	w2_vsig=dexp((vsig)**2)*(vsig-dawson(vsig))/dsqrt(pi)	
	fnormp=0.5d0+k2_vinf+fac_b*w2_vsig			


c	-----------------------------------------------------------------	
c	Calculation of the zero current equation (flux_e-flux_p) at 
c	r=r_max.  The result is stored in the first term of the two 
c	dimensional vector fn.
c	-----------------------------------------------------------------	
	

	flux_e=(0.25d0/fnorme)*(r_exo**2)*dsqrt(((2.d0*pk-3.d0)*bk*T0e)
     ^	/me)*(ak/(pk*(pk-1.d0)))*(1+x(2))*(1+(x(2)/pk))**(-pk)
     
     	flux_p=dsqrt((bk*T0p)/(2.d0*pi*mp))*((r_max**2)*dexp(-x(1))-
     ^	((r_max**2)-(r_exo**2))*dexp(-hp))/fnormp
     
     	fn(1)=flux_e-flux_p
     
	
c	-----------------------------------------------------------------	
c	Calculation of the quasi-neutrality equation (den_e-den_p) at 
c	r=r_max.  The result is stored in the second term of the two 
c	dimensional vector fn.
c	-----------------------------------------------------------------


	den_e=(((1.d0+(ge_rm/pk))**(-pk-1.d0))*((d**1.5d0)*(1.d0-0.5d0*
     ^	beta2b)-0.5d0*alpha*(c**1.5d0)*beta2f))/fnorme
     
     	den_p=(0.5d0*(dexp(-x(1))-dsqrt(1.d0-(r_exo/r_max)**2)*dexp(-hp)))
     ^	/fnormp
     
	fn(2)=den_e-den_p	
	
	return
	end
	
	
	
c	*******************************************************************
c	*******************************************************************
c	*******************************************************************


c	-------------------------------------------------------------------
c	This subroutine calculates den_p-den_e and their derivates.
c	-------------------------------------------------------------------	
	
	subroutine qn_alt(gp,xx,rr,r_max,dich,fn_qn,dfn_qn)
	implicit real*8 (a-h,o-z)
	dimension xx(2)
	integer dich
	real*8 n0,np_gp,ne_gp,mu
	real*8 funcd,zzk,gammln
	real*8 inf,m_sun,mp,me,L	
	parameter(n0=3.d10)
	COMMON/consts/inf,bk,ec,m_sun,mp,me,g,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp

	pi=4.d0*datan(1.d0)
	L=(m_sun*mp*G)/(r_exo*r_sun*bk*T0p)
	eta=(r_exo/rr)**2
	eta_m=(r_exo/r_max)**2
	mu=(r_max/rr)**2	

	hp_rm=xx(1)/(1.d0-eta_m)
	hp_rr_rm=(mu*xx(1)-gp)/(mu-1.d0)
	hp_rr_r0=gp/(1.d0-eta)

c	hp_rr_rm=((r_max**2)*xx(1)-(rr**2)*gp)/((r_max**2)-(rr**2))
c	hp_rr_r0=-(rr**2)*gp/((r_exo**2)-(rr**2))
c	Formalisme de Jockers abandonne ici (modifications 09/01/07)
c	Les resultats restent identiques, seul le formalisme change
c	(eta et mu apparaissent explicitement dans hp_rr_rm et hp_rr_r0).
	
	if (rr. gt. r_max) then
	np_gp=0.5d0*n0*(dexp(-xx(1))*funcd(xx(1)-gp)-dsqrt(1.d0-eta)*
     ^	dexp(-hp_rm)*funcd(hp_rm-hp_rr_r0)-dsqrt(1.d0-mu)*(dexp(-xx(1))*
     ^	funcd(xx(1)-hp_rr_rm)-dexp(-hp_rm)*funcd(hp_rm-hp_rr_rm)))
     	else
	np_gp=0.5d0*n0*(2.d0*dexp(-gp)-dexp(-xx(1))*funcd(xx(1)-gp)-
     ^	dsqrt(1.d0-eta)*(2.d0*dexp(-hp_rr_r0)-dexp(-hp_rm)*
     ^	funcd(hp_rm-hp_rr_r0))+dsqrt(mu-1.d0)*(funcc(hp_rr_rm-hp_rm)*
     ^	dexp(-hp_rm )-funcc(hp_rr_rm-xx(1))*dexp(-xx(1))))
        endif   
     
     	np_gp=np_gp/fnormp	


	ge=L*(T0p/T0e)*(1.d0+(me/mp))*(1.d0-(r_exo/rr))-gp*(T0p/T0e)
	ge=ge*(pk/(pk-1.5d0))
	
	if (ge. lt. 0.d0) then
	fn_qn=inf
	dfn_qn=0.d0
	return
	endif 
		
	fac_ne=(1.d0+(ge/pk))**(-pk-1)
	eta=(r_exo/rr)**2
	d=1.d0+(ge/pk)
	b=1.d0/(1.d0+((xx(2)-ge)/(pk+ge)))
	alpha=dsqrt(1.d0-eta)*(1.d0+(eta/(1.d0-eta))*
     ^	(ge/(pk+ge)))**(-pk-1.d0)
         
	c=1.d0+(ge/((1.d0-eta)*pk))
	f=1.d0/(1.d0+((1.d0-eta)*xx(2)-ge)/((1.d0-eta)*pk+ge))

     	if (b .gt. 1.d0) then 
	b=1.d0
	elseif (b .lt. 0.d0) then
	b=0.d0
	endif
	if (f .gt. 1.d0) then 
	f=1.d0
	elseif (f .lt. 0.d0) then
	f=0.d0
	endif	     
     
     	beta2b=zzk(pk-0.5d0,1.5d0,b)
	beta2f=zzk(pk-0.5d0,1.5d0,f)
		
	ne_gp=n0*fac_ne*((d**1.5d0)*(1.d0-0.5d0*beta2b)-0.5d0*
     ^	alpha*(c**1.5d0)*beta2f)   
     
     	ne_gp=ne_gp/fnorme

	fn_qn=np_gp-ne_gp
	
	
c	---------------------------------------------------------------	
c	Test on integer 'dich' : if 'dich' is equal to 1, a dichotomy
c	method is used and the calculation of the derivative of the
c	quasi-neutrality equation, dfn_qn, is not necessary.  Instead
c	dfn_qn is set equal to 0.  If 'dich' is equal to 0, then 
c	dfn_qn is calculated analytically.
c	--------------------------------------------------------------- 	
	
	if (dich .eq. 1) then
	dfn_qn=0.d0
	else
     
	if (rr. gt. r_max) then     
     	dnp_gp=0.5d0*n0*(dexp(-xx(1))*(dsqrt(rr**2/(rr**2-r_max**2))*
     ^	funcd(xx(1)-hp_rr_rm)-funcd(xx(1)-gp))+dexp(-hp_rm)*(dsqrt
     ^	(rr**2/(rr**2-r_exo**2))*funcd(hp_rm-hp_rr_r0)-dsqrt(rr**2/
     ^	(rr**2-r_max**2))*funcd(hp_rm-hp_rr_rm)))
     	else
	if (gp .lt. (xx(1)*(1.d0-eta)/(1.d0-eta_m))) then
	dnp_gp=0.5d0*n0*(-2.d0*dexp(-gp)+dexp(-xx(1))*(funcd(xx(1)-
     ^	gp)-(1.d0/dsqrt(pi))*(1.d0/dsqrt(xx(1)-gp)))-(1.d0/dsqrt(1.d0
     ^	-eta))*(-2.d0*dexp(-hp_rr_r0)+dexp(-hp_rm)*(funcd(hp_rm-hp_rr_r0)
     ^	-(1.d0/dsqrt(pi))*(1.d0/dsqrt(hp_rm-hp_rr_r0))))-(1.d0/dsqrt(
     ^	mu-1.d0))*(dexp(-hp_rm)*(-funcc(hp_rr_rm-hp_rm)+(1.d0/dsqrt(pi))*
     ^	(1.d0/dsqrt(hp_rr_rm-hp_rm)))-dexp(-xx(1))*(-funcc(hp_rr_rm-
     ^	xx(1))+(1.d0/dsqrt(pi))*(1.d0/dsqrt(hp_rr_rm-xx(1))))))
     	else
	dnp_gp=0
	endif
	
     	endif
     
     	dnp_gp=dnp_gp/fnormp
	
	db=1.d0/(pk+xx(2))
	df=1.d0/((1.d0-eta)*(xx(2)+pk))
	dalpha=-(pk*(pk+1.d0)*eta*alpha)/(((1.d0-eta)*pk+ge)*(pk+ge))
	ak=dexp(gammln(pk+1.d0)-gammln(pk-0.5d0)-gammln(1.5d0))
	dbeta2b=ak*(b**1.5d0)*dsqrt(1.d0-b)
	dbeta2f=ak*(f**1.5d0)*dsqrt(1.d0-f)
	
	dne_gp=(ne_gp*fnorme)*((pk+1.d0)/(pk+ge))+n0*fac_ne*((1.5d0*
     ^	dsqrt(d)*(1.d0-0.5d0*beta2b)/pk)-(0.5d0*(d**1.5d0)*dbeta2b*db)-
     ^	(0.5d0*(c**1.5d0)*beta2f*dalpha)-(0.75d0*alpha*beta2f*dsqrt(c)/
     ^	((1.d0-eta)*pk))-(0.5d0*alpha*(c**1.5d0)*dbeta2f*df))
     
     	dne_gp=dne_gp/fnorme
	
	dfn_qn=dnp_gp-dne_gp
	
	endif
	
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************
	subroutine solarwind(xx,r_max,alt_crit,gp_sol)
        implicit real*8(A-H,O-Z)
	parameter(nbiter=200)
	real*8 inf,m_sun,mp,me	
	COMMON/consts/inf,bk,ec,m_sun,mp,me,g,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp
	integer dich	
	dimension gp_sol(nbiter),sol(3),xx(2)
	
    	pi=4.d0*datan(1.d0)            	
	
	i=1
	do j=1,nbiter
	alt=r_exo+(j/dfloat(nbiter))*(r_end-r_exo)
	
	if (alt .lt. r_max) then
	
	gp_min=0.d0
	gp_max=((nint(100.d0*xx(1)))/100.d0)
	gp_old=gp_min
	fn_qn_old=1000.d0
	step_gp=0.01d0	
	
	k=0
	do gp=gp_min,gp_max,step_gp
	dich=1	
	call qn_alt(gp,xx,alt,r_max,dich,fn_qn,dfn_qn)

	if (fn_qn .eq. inf) goto 210
	
	if ((fn_qn*fn_qn_old) .lt. 0.d0) then 
	gp_inf1=gp_old
	gp_sup1=gp
	if (gp_inf1 .eq. gp_sup1) then
	goto 208
	else
	k=k+1
	dich=0
	sol(k)=rtsafe(xx,alt,r_max,dich,gp_inf1,gp_sup1,1.d-9)
	endif
	else
	endif
208	fn_qn_old=fn_qn
	gp_old=gp
	
	enddo
c	end gp loop (alt < r_max)			
	
	else
	
	gp_min=-80.d0
	gp_max=((nint(100.d0*xx(1)))/100.d0)+1.00d0
	gp_old=gp_max
	fn_qn_old=1000.d0
	step_gp=-0.01d0	
	
	k=0
	do gp=gp_max,gp_min,step_gp
	dich=1
	call qn_alt(gp,xx,alt,r_max,dich,fn_qn,dfn_qn)

c	if (alt .lt. 6.58d0 .and. alt .gt. 6.57d0) then
c	write(100,*) gp,fn_qn
c	endif
	
	if ((fn_qn*fn_qn_old) .lt. 0.d0 .and. fn_qn_old .ne. inf) then 
	gp_inf1=gp_old
	gp_sup1=gp
	if (gp_inf1 .eq. gp_sup1) then
	goto 209
	else
	k=k+1
	dich=0
	sol(k)=rtsafe(xx,alt,r_max,dich,gp_inf1,gp_sup1,1.d-9)
	endif
	else
	endif	
209	fn_qn_old=fn_qn
	gp_old=gp	
	
	enddo
c	end loop gp (alt > r_max)
	
	endif
	
210	if (k .eq. 3) then
	if (alt .lt. alt_crit) then
	gp_sol(i)=sol(2)
	else
	gp_sol(i)=sol(3)
	endif
	else
	if (alt .lt. alt_crit .and. alt .gt. r_max) then
	call trick_rmax(xx,alt,r_max,gp_max,gp_min,step_gp,dich,gp_trick)
	gp_sol(i)=gp_trick
	else
	gp_sol(i)=sol(1)
	endif
	endif
			
	i=i+1	

	enddo
c	end loop alt

	return
	end	

c	****************************************************************
c	****************************************************************
c	****************************************************************

	subroutine trick_rmax(xx,alt,r_max,gp_max,gp_min,step_gp,dich,gp_trick)
	implicit real*8(a-h,o-z)
	real*8 inf,bk,ec,m_sun,mp,me,G,r_sun
	COMMON/consts/inf,bk,ec,m_sun,mp,me,G,r_sun
	integer dich
	dimension xx(2)

	gp_old=gp_max
	fn_qn_old=-1.d20
	step_gp=-0.01d0	
	do gp=gp_max,gp_min,step_gp
	dich=1
	call qn_alt(gp,xx,alt,r_max,dich,fn_qn,dfn_qn)
	if (fn_qn .eq. inf) goto 213
	if (fn_qn .lt. fn_qn_old) then
c	write(6,*) 'test', fn_qn_old, fn_qn
c	write(6,*) 'Radial distance', alt, ' : gp = ', gp
	gp_trick=gp
	return
	endif
	fn_qn_old=fn_qn
213	gp_old=gp
	enddo		

	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************

	subroutine moments(ident,rr,r_max,pot_exo,pot_max,pot_r,xx,results)
	implicit real*8(A-H,O-Z)
	parameter(nbiter=200, den0=3.d10)
	real*8 inf,m_sun,mp,me
	real*8 L,Le,mu	
	COMMON/consts/inf,bk,ec,m_sun,mp,me,g,r_sun
	COMMON/param/r_exo,T0e,T0p,pk,r_end
	COMMON/fnorm/fnorme,fnormp
	dimension results(5),xx(2)
	
	pi=4.d0*datan(1.d0)
	
	L=(m_sun*mp*G)/(r_exo*r_sun*bk*t0p)
	Le=L*(me/mp)*(T0p/T0e)

	y=r_exo/rr
	y_m=r_exo/r_max	
	mu=(r_max/rr)**2.0d0
	sigma=mu-1.d0
	eta=(r_exo/rr)**2.0
	p=1.d0-eta
	eta_m=(r_exo/r_max)**2.0d0
	

c	------------------------------------------------------------------	
c	Here we calculate the moments of order 0,1,2 and 3 of the
c	velocity distribution functions of electrons (parameter ident=1).
c	Since the electrons are always located in an attractive potential,
c	these moments have to be calculated for different classes of
c	particles, namely ballastic, trapped and escaping electrons.
c	The incoming electrons are assumed to be empty.  This is true 
c	from the exobase r_exo to infinity. 
c
c	The formulae used below are those described in Pierrard & Lemaire,
c	JGR 101, 7923, 1996 for a kappa distribution. 	
c	------------------------------------------------------------------
	
	if (ident .eq. 1) then
	
	ak=dexp(gammln(pk+1.d0)-gammln(pk-0.5d0)-gammln(1.5d0))
	ak2=dexp(gammln(pk+1.d0)-gammln(pk-1.5d0)-gammln(2.5d0))	

	alphae=(ec*pot_exo*r_exo*r_sun)/(me*g*m_sun)
      	betae=(ec*pot_r*rr*r_sun)/(me*g*m_sun)

	qe=(Le*((1.d0-y)+(alphae-betae*y)))*(pk/(pk-1.5d0))
	qmve=Le*(1.d0+alphae)*(pk/(pk-1.5d0))
	
c	vinfsq=(le*(1.d0+betae)*y)*(pk/(pk-1.5d0))
	vinfsq=qmve-qe	
	xsq=p*vinfsq-eta*qe

	bj=1.d0/(1.d0+vinfsq/(pk+qe))
	if (bj .gt. 1.0d0 .or. bj .lt. 0.0d0) then
	 if (bj. lt .0.0d0) then
	 fcn1=0.0d0
	 fc1=0.0d0
	 endif
	 if (bj. gt. 1.0d0) then
	 fcn1=1.0d0
	 fc1=1.0d0
	 endif
	else
      	 fcn1=zzk(pk-0.5d0,1.5d0,bj)
	 fc1=zzk(pk-1.5d0,2.5d0,bj)
	endif

 	if (p.gt.0.d0) then
      	f=1.d0/(1.d0+xsq/(p*pk+qe))
	if (f .gt. 1.0d0 .or. f .lt. 0.0d0) then
	 if (f .lt. 0.0d0) then
	 fcn2=0.0d0
	 fc2=0.0d0
	 endif
	 if (f .gt. 1.0d0) then
	 fcn2=1.0d0
	 fc2=1.0d0
	 endif
	else
	 fcn2=zzk(pk-0.5d0,1.5d0,f)
	 fc2=zzk(pk-1.5d0,2.5d0,f)
	endif
        else
	f=0.d0
	fcn2=0.0d0
	fc2=0.0d0
	endif
	
	facden=den0*(1.d0+qe/pk)**(-pk-1.d0)
	fac=facden/pk**1.5d0
	d2=(1.d0+qe/pk)**2.5d0
	
	if((p .gt. 0.d0).and.(p*pk .ge. -qe))then
	 ccm=((p*pk+qe)**(1.5d0))/p
	 cm=(1.d0+qe/(p*pk))**2.5d0
	 cm2=(1.d0+qe/(p*pk))**1.5d0
	 eqp=eta*qe/p
       	 aam=(1.d0+eqp/(pk+qe))**(-pk-1.d0)		
	else
	 aam=0.d0
	 ccm=0.d0
	 cm=0.d0
	 cm2=0.d0
	endif	
	
	w=dsqrt((2.d0*pk-3.d0)*bk*T0e/(me*pk))
	coef=(1.0d0/6.0d0)*me*w*w*pk*(ak/ak2)*facden 
	coefchal=(1.0d0/8.0d0)*me*w**3*dsqrt(pk)	

c			********************
c			ELECTRONS DENSITIES
c			********************	


c 	Ballistic electrons
c	-------------------

	denbal=fac*(((pk+qe)**1.5d0)*(1.d0-fcn1)-
     ^  aam*ccm*(1.d0-fcn2))
         
c	Trapped electrons
c	-----------------

	dentrp=fac*aam*ccm*(1.d0-fcn2)
	
c 	Escaping electrons
c	------------------

	denesc=0.5d0*fac*(((pk+qe)**1.5d0)*fcn1-aam*ccm*fcn2)

c       Total electron density
c       ----------------------

      	den=(denbal+dentrp+denesc)/fnorme
     	

c			*********************
c			ELECTRONS FLUX MASSES
c			*********************
    
    
c       Escaping electrons
c       ------------------

	flux=0.25d0*eta*den0*ak*w*dsqrt(pk)*((1.d0+qmve/pk)**(-pk))*
     ^  (1.d0+qmve)/(pk*(pk-1.d0))/fnorme

	
c			**********************	
c			ELECTRONS TEMPERATURES
c			**********************


c       Ballistic electrons
c       -------------------
	
	pparbal=2.0d0*coef*(d2*(1.0d0-fc1)-aam*(p**1.5d0)*cm*(1.0d0-fc2))
	pperbal=2.0d0*coef*(d2*(1.0d0-fc1)-aam*cm*(1.0d0+eta/2.0d0)*
     ^	dsqrt(p)*(1.0d0-fc2)-(1.5d0/pk)*eta*(qe/dsqrt(p))*aam*cm2*
     ^	(1.0d0-fcn2)*(ak2/ak))
    
c       Trapped electrons
c       -----------------

	ppartpd=2.0d0*coef*(aam*cm*(p**1.5d0)*(1.0d0-fc2))
	ppertpd=2.0d0*coef*(aam*cm*dsqrt(p)*(1.0d0+eta/2.0d0)*
     ^  (1.0d0-fc2)+(1.5d0/pk)*eta*(qe/dsqrt(p))*aam*cm2*(1.0d0-fcn2)
     ^  *(ak2/ak))
     
c       Escaping electrons
c       ------------------

	pparesc=coef*(d2*fc1-aam*cm*(p**1.5d0)*fc2)
	pperesc=coef*(d2*fc1-aam*cm*dsqrt(p)*(1.0d0+eta/2.0d0)*fc2-
     ^  (1.5d0/pk)*eta*(qe/dsqrt(p))*aam*cm2*fcn2*(ak2/ak))

c 	All electrons
c       -------------

	ppar=(pparbal+pparesc+ppartpd)/fnorme
	pper=(pperbal+pperesc+ppertpd)/fnorme
		
	
c			********************	
c			ELECTRONS HEAT FLUX
c			********************

	if (pk .ne. 2.0d0) then

	heatflux=eta*den0*ak*coefchal*(1.d0+qmve/pk)**(-pk)*(pk/
     ^  (pk-2.d0)*(1.d0+qmve/pk)**2-2.d0*pk/(pk-1.d0)*(1.d0+qmve/pk)+
     ^  1.d0-(qe/w**2)*(1.d0+qmve)/(pk*(pk-1.d0)))/fnorme 
     
     	else
	
	heatflux=0.d0
	
	endif
     
     	else
	
c	------------------------------------------------------------------	
c	Here we calculate the moments of order 0,1,2 and 3 of the
c	velocity distribution functions of protons (parameter ident=2).
c	Since the protons are located in an attractive potential between
c	r_exo and r_max, these moments have to be calculated for different 
c	classes of particles, namely ballastic, trapped and escaping 
c	protons.  The incoming protons are assumed to be empty.  Beyond
c	r_max, the protons are located in a repulsive potential and 
c	the moments have to be calculated only for escaping particles.
c
c	The two sets of formulae used below are those described in Lamy 
c	et al., JGR 108, 1047, 2003. 	
c	------------------------------------------------------------------	
	
	
	if (rr .lt. r_max) then
	
c	--------------------------------------------------------	
c	rr is larger than r_max : the protons are located in a
c	repulsive potential.
c	--------------------------------------------------------	
	
	alphap=(-ec*pot_exo*r_exo*r_sun)/(mp*g*m_sun)
	beta_m=(-ec*pot_max*r_max*r_sun)/(mp*g*m_sun)
	betap=(-ec*pot_r*rr*r_sun)/(mp*g*m_sun)
	
	qmvp=L*((1.d0+alphap)-(1.d0+beta_m)*y_m)
	qp=L*((1.d0-y)+(alphap-betap*y))
	vinfsq=L*((1.d0+betap)*y-(1.d0+beta_m)*y_m)
	
	eqp=eta*qp/p	
	coef=sigma*eta_m/(1.d0-eta_m)
        xsq=-L*(((1.d0+beta_m)*y_m)*(1.d0-coef)+coef*
     ^	(1.d0+alphap)-(1.d0+betap)*y)
	fac_a=dsqrt(p)*dexp(-eqp)
	fac_b=dsqrt(sigma)*dexp(-(mu*vinfsq/sigma))
	c0=dsqrt(8.0d0*bk*T0p/(pi*mp))
	p0=den0*bk*T0p
	
	if (vinfsq .lt. 0.d0) then
	vinf=0.d0
	vsig=0.d0
	else
	vinf=dsqrt(vinfsq)
	vsig=dsqrt(vinfsq/sigma)
	endif
	
	if (xsq .lt. 0.d0) then
	xsig=0.d0
	xp=0.d0
	else
	xsig=dsqrt(xsq/sigma)
	xp=dsqrt(xsq/p)
	endif

c			******************
c			PROTONS DENSITIES
c			******************
	
	
c  	ballistic protons
c       -----------------

	denbal=den0*dexp(-qp)*(1.d0-erfcc(vinf)-fac_a*(1.d0-
     ^	erfcc(xp))-(2.d0/dsqrt(pi))*fac_b*(dexp(vinfsq/sigma)*
     ^	dawson(vsig)-dexp(xsq/sigma)*dawson(xsig)))
		
c 	escaping protons
c       ----------------
 
	denesc=0.5d0*den0*dexp(-qp)*(erfcc(vinf)-fac_a*erfcc(xp)-
     ^	(2.d0/dsqrt(pi))*fac_b*(dexp(xsq/sigma)*dawson(xsig)-
     ^  dexp(vinfsq/sigma)*dawson(vsig)))

c  	trapped protons
c       ---------------

	if (xp .gt. 0.d0) then
	dentrp=den0*dexp(-qp)*(fac_a*(1.d0-erfcc(xp))-(2.d0/dsqrt(pi))*
     ^	fac_b*dexp(xsq/sigma)*dawson(xsig))
     	else
	dentrp=0.d0
	endif
	
c 	Total protons density
c       ---------------------

	den=(denbal+denesc+dentrp)/fnormp
	

c			********************
c			PROTONS MASS FLUXES
c			*******************


	flux=0.25d0*den0*c0*(mu*dexp(-qmvp)+(eta-mu)*dexp(-qmvp/
     ^	(1.d0-eta_m)))/fnormp
    
c			********************    
c			PROTONS TEMPERATURES
c			********************
 
	
c 	ballistic protons
c       -----------------

	pparbal=(denbal*bk*T0p)+p0*dexp(-qp)*(eta*fac_a*(1.d0-erfcc(xp)
     ^	-(2.d0/dsqrt(pi))*xp*dexp(-xsq/p))+mu*fac_b*(2.d0/dsqrt(pi))*
     ^	(dexp(vinfsq/sigma)*(dawson(vsig)-vsig)-dexp(xsq/sigma)*
     ^	(dawson(xsig)-xsig)))
     
    	pperbal=1.5d0*denbal*bk*T0p-0.5d0*pparbal-p0*dexp(-qp)*
     ^	(eqp*fac_a*(1.d0-erfcc(xp))+(2.d0/dsqrt(pi))*(mu*vinfsq/sigma)*
     ^	fac_b*(dexp(vinfsq/sigma)*dawson(vsig)-dexp(xsq/sigma)*
     ^	dawson(xsig)))
     
	
c	escaping protons
c       ----------------
 
	pparesc=(denesc*bk*t0p)+0.5d0*(den0*bk*t0p)*dexp(-qp)*(eta*
     ^	fac_a*(erfcc(xp)+(2.d0/dsqrt(pi))*xp*dexp(-xsq/p))-(2.d0/
     ^	dsqrt(pi))*mu*fac_b*(dexp(vinfsq/sigma)*(dawson(vsig)-vsig)-
     ^	dexp(xsq/sigma)*(dawson(xsig)-xsig)))
    
	pperesc=1.5d0*denesc*bk*T0p-0.5d0*pparesc-0.5d0*p0*dexp(-qp)*
     ^	(eqp*fac_a*erfcc(xp)-(2.d0/dsqrt(pi))*(mu*vinfsq/sigma)*
     ^	fac_b*((dexp(vinfsq/sigma))*dawson(vsig)-(dexp(xsq/sigma))*
     ^  dawson(xsig)))
     
		
c 	trapped protons
c       ---------------

	ppartrp=(dentrp*bk*T0p)-(den0*bk*T0p)*dexp(-qp)*(eta*fac_a*
     ^	(1.d0-erfcc(xp)-(2.d0/dsqrt(pi))*xp*dexp(-xsq/p))+(2.d0/
     ^	dsqrt(pi))*mu*fac_b*dexp(xsq/sigma)*(dawson(xsig)-xsig))

	ppertrp=1.5d0*dentrp*bk*T0p-0.5d0*ppartrp+p0*dexp(-qp)*
     ^	(eqp*fac_a*(1.d0-erfcc(xp))-(2.d0/dsqrt(pi))*
     ^	(mu*vinfsq/sigma)*fac_b*dexp(xsq/sigma)*dawson(xsig))
     

c       **************	
c 	all particles
c       **************
	            	
	ppar=(pparbal+pparesc+ppartrp)/fnormp
	pper=(pperbal+pperesc+ppertrp)/fnormp 
	
	
c			*****************	
c			PROTONS HEAT FLUX
c			*****************	
	
	heatflux=0.25d0*p0*c0*dexp(-qmvp)*(mu*(2.d0+qmvp-qp)-
     ^	((2.d0-qp)*(mu-eta)+mu*qmvp)*dexp(-eta_m*qmvp/(1.d0-eta_m)))
     
     	heatflux=heatflux/fnormp
     
	else
	
c	-------------------------------------------------------------------	
c	rr is larger than r_max : the protons are located in a repulsive
c	potential.  All protons that have reached r_max are escaping 
c	particles.
c	-------------------------------------------------------------------	
	
	alphap=(-ec*pot_exo*r_exo*r_sun)/(mp*g*m_sun)	
	beta_m=(-ec*pot_max*r_max*r_sun)/(mp*g*m_sun)
	betap=(-ec*pot_r*rr*r_sun)/(mp*g*m_sun)
	
	qp=L*((1.d0-y)+(alphap-betap*y))		
	qmvp=L*((1.d0+alphap)-(1.d0+beta_m)*y_m)
	
	eqp=eta*qp/p
	fac_a=dsqrt(p)*dexp(-eqp)
	fac_b=dsqrt(1.d0-mu)*dexp(mu*(qmvp-qp)/(1.d0-mu))
	p0=den0*bk*t0p
	c0=dsqrt(8.d0*bk*t0p/(mp*pi))  	

	if((qmvp-qp) .gt. 0.d0) then
	vprim_m=dsqrt(qmvp-qp)
	else
	vprim_m=0.d0
	endif
	y_m2=(qmvp/(1.d0-eta_m))-(qp/(1.d0-eta)) 
	if(y_m2 .gt. 0.d0) then
	y_m=dsqrt(y_m2)
	else
	y_m=0.d0
	endif
	x_m2=((qmvp-qp)/(1.d0-mu))+(eta_m*qmvp)/(1.d0-eta_m)	
	if(x_m2 .gt. 0.d0) then
	x_m=dsqrt(x_m2)
	else
	x_m=0.d0
	endif
	if(qmvp-qp .gt. 0.d0)then    
	xprim_m=vprim_m/dsqrt(1.d0-mu)
	else
	xprim_m=0.d0
	endif

c			***************** 	
c			PROTONS DENSITIES
c			*****************

	den=0.5d0*den0*dexp(-qp)*(erfcc(vprim_m)-fac_a*
     ^	erfcc(y_m)-fac_b*(erfcc(xprim_m)-erfcc(x_m)))/fnormp

	
c			*****************	
c			PROTONS MASS FLUX
c			*****************
    
	flux=0.25d0*den0*c0*((mu*dexp(-qmvp))-(mu-eta)*
     ^	dexp(-qmvp/(1.d0-eta_m)))/fnormp 
     
     
c			*********************     
c			PROTONS TEMPERATURES
c			********************
 
 	ppar_p=(den*fnormp*bk*T0p)+0.5d0*(den0*bk*T0p)*dexp(-qp)*
     ^	(eta*fac_a*(erfcc(y_m)+(2.d0/dsqrt(pi))*y_m*dexp(-y_m2))+
     ^	mu*fac_b*(erfcc(xprim_m)+(2.d0/dsqrt(pi))*xprim_m*
     ^	dexp(-(xprim_m**2))-erfcc(x_m)-(2.d0/dsqrt(pi))*x_m*
     ^	dexp(-x_m2)))
     
     	ppar=ppar_p/fnormp
	
	pper_p=(1.5d0*den*fnormp*bk*T0p)-0.5d0*ppar_p-0.5d0*p0*dexp(-qp)*
     ^  (eqp*fac_a*erfcc(y_m)-(mu*(vprim_m**2)/(1.d0-mu))*fac_b*
     ^  (erfcc(xprim_m)-erfcc(x_m)))
     
     	pper=pper_p/fnormp	
	    
	
c			*****************	
c			PROTONS HEAT FLUX
c			*****************	
	
	heatflux=0.25*p0*c0*dexp(-qmvp)*(mu*(2.d0+qmvp-qp)-
     ^	((2.d0-qp)*(mu-eta)+mu*qmvp)*dexp(-eta_m*qmvp/(1.d0-eta_m)))
     
     	heatflux=heatflux/fnormp
     	
	endif
	
c	-------------------------------------------------------------	
c	End of if test on rr lower or larger than r_max, i.e. end
c	of the test to know if the protons are in an attractive or
c	in a repulsive potential
c	-------------------------------------------------------------	
	
	endif 
	
c	-------------------------------------------------------------	
c	End of if test on type particles : either electrons (ident=1)  
c	or protons (ident=2)	
c	-------------------------------------------------------------

c	-------------------------------------------------------------     
c	The moments for electrons or for protons are written in a 
c	vector called results that is transferred to the main program.
c       The vector contains in order the density, the mass flux, the 
c	parallel and perpendicular pressures and the heat flux.
c	-------------------------------------------------------------     
     
     	results(1)=den
	results(2)=flux
	results(3)=ppar
	results(4)=pper
	results(5)=heatflux      

	return
	end

c	****************************************************************
c	****************************************************************
c	****************************************************************	
		
	
     	FUNCTION rtsafe(xx,alt,r_max,dich,x1,x2,xacc)
	implicit real*8(a-h,o-z)
	dimension xx(2)
      	INTEGER MAXIT,j,dich
      	PARAMETER (MAXIT=100)
      	call qn_alt(x1,xx,alt,r_max,dich,fl,df)
      	call qn_alt(x2,xx,alt,r_max,dich,fh,df)
      	if((fl.gt.0. .and. fh.gt.0.).or.(fl.lt.0. .and. fh.lt.0.))pause
     *	'root must be bracketed in rtsafe'
      	if(fl.eq.0.)then
        rtsafe=x1
        return
      	else if(fh.eq.0.)then
        rtsafe=x2
        return
      	else if(fl.lt.0.)then
        xl=x1
        xh=x2
      	else
        xh=x1
        xl=x2
      	endif
      	rtsafe=.5*(x1+x2)
      	dxold=dabs(x2-x1)
      	dx=dxold
      	call qn_alt(rtsafe,xx,alt,r_max,dich,f,df)
      	do 11 j=1,MAXIT
        if(((rtsafe-xh)*df-f)*((rtsafe-xl)*df-f).ge.0. .or. dabs(2.*
     *	f).gt.dabs(dxold*df) ) then
          dxold=dx
          dx=0.5*(xh-xl)
          rtsafe=xl+dx
          if(xl.eq.rtsafe)return
        else
          dxold=dx
          dx=f/df
          temp=rtsafe
          rtsafe=rtsafe-dx
          if(temp.eq.rtsafe)return
        endif
        if(dabs(dx).lt.xacc) return
        call qn_alt(rtsafe,xx,alt,r_max,dich,f,df)
        if(f.lt.0.) then
          xl=rtsafe
        else
          xh=rtsafe
        endif
11    	continue
      	pause 'rtsafe exceeding maximum iterations'
      	return
      	END				
	
c	*******************************************************************
c	*******************************************************************
c	*******************************************************************	
	
	subroutine ludcmp(a,n,np,indx,d)
	integer n,np,indx(n),nmax
	real*8 d,a(np,np),tiny
	parameter (nmax=500,tiny=1.0d-20)	
	integer i,imax,j,k
	real*8 aamax,dum,sum,vv(nmax)
	
	d=1.d0
	do i=1,n
	aamax=0.
	do j=1,n
	if (abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
	enddo 
	if (aamax.eq.0) pause 'singular matrix in ludcmp'
	vv(i)=1./aamax
	enddo
	do j=1,n
	do i=1,j-1
	sum=a(i,j)
	do k=1,i-1
	sum=sum-a(i,k)*a(k,j)
	enddo 
	a(i,j)=sum
	enddo 
	aamax=0.
	do i=j,n
	sum=a(i,j)
	do k=1,j-1
	sum=sum-a(i,k)*a(k,j)
	enddo
	a(i,j)=sum
	dum=vv(i)*abs(sum)
	if (dum.ge.aamax) then
	imax=i
	aamax=dum
	endif
	enddo
	if (j.ne.imax) then
	do k=1,n
	dum=a(imax,k)
	a(imax,k)=a(j,k)
	a(j,k)=dum
	enddo
	d=-d
	vv(imax)=vv(j)
	endif
	indx(j)=imax
	if (a(j,j).eq.0.) a(j,j)=tiny
	if (j.ne.n) then
	dum=1./a(j,j)
	do i=j+1,n
	a(i,j)=a(i,j)*dum
	enddo
	endif
	enddo 
	return
	end
	
c	******************************************************************
c	******************************************************************
c	******************************************************************

	subroutine lubksb(a,n,np,indx,b)
	integer n,np,indx(n)
	real*8 a(np,np),b(n)
	integer i,ii,j,ll
	real*8 sum
	
	ii=0
	do i=1,n
	ll=indx(i)
	sum=b(ll)
	b(ll)=b(i)
	if (ii.ne.0) then
	do j=ii,i-1
	sum=sum-a(i,j)*b(j)
	enddo 
	else if (sum.ne.0.) then
	ii=i
	endif
	b(i)=sum
	enddo
	do i=n,1,-1
	sum=b(i)
	do j=i+1,n
	sum=sum-a(i,j)*b(j)
	enddo 
	b(i)=sum/a(i,i)
	enddo 
	return
	end
	
c 	*****************************************************************		
c	*****************************************************************
c	*****************************************************************

	subroutine fdjac(n,x,r_max,fvec,np,df)
	integer n,np,nmax
	integer i,j	
	real*8 df(np,np),fvec(n),x(n),eps,r_max
	parameter(nmax=40,eps=1.d-4)
	real*8 h,temp,f(nmax)	
	do j=1,n
	temp=x(j)
	h=eps*dabs(temp)
	if (h.eq.0.d0) h=eps
	x(j)=temp+h
	h=x(j)-temp	
	call funcv(n,x,r_max,f) 
	x(j)=temp
	do i=1,n
	df(i,j)=(f(i)-fvec(i))/h
	enddo
	enddo
	return
	end	
	
c 	*****************************************************************		
c	*****************************************************************
c	*****************************************************************	
	
    	function zzk(a,b,x) 
	double precision zzk,a,b,x
	double precision bt, betacf,gammln
	
	if(x.lt.0.d0.or.x.gt.1.d0)write(6,*)'bad argument x in zzk'
	if(x.eq.0.d0.or.x.eq.1.d0) then
		bt=0.d0
	else
	   bt=dexp(gammln(a+b)-gammln(a)-gammln(b)
     ^     +a*dlog(x)+b*dlog(1.d0-x))
	endif
	if(x.lt.(a+1.d0)/(a+b+2.d0))then
		zzk=bt*betacf(a,b,x)/a
		return
	else
		zzk=1.d0-bt*betacf(b,a,1.d0-x)/b	
		return
	endif
        end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************

	function betacf(a,b,x)
	integer maxit
     	double precision a,b,x
	double precision betacf
	double precision eps,fpmin
	parameter (maxit=100,eps=3.e-7,fpmin=1.e-30)
	integer m, m2
	double precision aa,c,d,del,h,qab,qam,qap
	qab=a+b
	qap=a+1.d0
	qam=a-1.d0
	c=1.d0
	d=1.d0-qab*x/qap
	if(dabs(d).lt.fpmin)d=fpmin
	d=1.d0/d
	h=d
	do m=1,maxit
	m2=2*m
	aa=m*(b-m)*x/((qam+m2)*(a+m2))
	d=1.d0+aa*d
	if (abs(d).lt.fpmin)d=fpmin
	c=1.d0+aa/c
	if(abs(c).lt.fpmin)c=fpmin
	d=1.d0/d
	h=h*d*c
	aa=-(a+m)*(qab+m)*x/((a+m2)*(qap+m2))
	d=1.d0+aa*d
	if(abs(d).lt.fpmin)d=fpmin
	c=1.d0+aa/c
	if(abs(c).lt.fpmin)c=fpmin
	d=1.d0/d
	del=d*c
	h=h*del
	if(abs(del-1.d0).lt.eps)goto 1
	enddo
	write(*,*)'a or b too big or maxit too small'
	stop
 1	betacf=h
	return
	end
			
c	****************************************************************
c	****************************************************************
c	****************************************************************

	function gammln(xy)
	real*8 gammln,xy	
	integer j
	real*8 ser,stp,tmp,xo,y,cof(6)
	data cof,stp/76.18009172947146d0,-86.50532032941677d0,
     ^   24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     ^  -.5395239384953d-5,2.5066282746310005d0/
	xo=xy
	y=xo
	tmp=xo+5.5d0
	tmp=(xo+0.5d0)*dlog(tmp)-tmp
	ser=1.000000000190015d0
	do j=1,6
	   y=y+1.d0
	   ser=ser+cof(j)/y
	enddo
	gammln=tmp+dlog(stp*ser/xo)
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
	FUNCTION GAMMP(a,x)
	real*8 a,gammp,x
	real*8 gammcf,gamser,gln
	if(x.lt.0.d0 .or. a.le.0.d0) then
	write(6,*) 'bad arguments in gammp'
	stop
	endif
	if(x.lt.a+1.d0) then
	call gser(gamser,a,x,gln)
	gammp=gamser
	else
	call gcf(gammcf,a,x,gln)
	gammp=1.d0-gammcf
	endif
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
	subroutine gser(gamser,a,x,gln)
	integer itmax
	real*8 a,gamser,gln,x,eps
	parameter(itmax=100,eps=3.d-7)
	integer n
	real*8 ap,del,sum,gammln
	gln=gammln(a)
	if(x.le.0.d0) then
	if(x.lt.0.d0) pause 'x<0 dans gser'
	gamser=0.
	return
	endif
	ap=a
	sum=1.d0/a
	del=sum
	do n=1,itmax
	ap=ap+1.d0
	del=del*x/ap
	sum=sum+del
	if (dabs(del).lt.dabs(sum)*eps) goto 12
	enddo
	write(6,*) 'a too large, itmax too small in gser'
	stop
12	gamser=sum*dexp(-x+a*dlog(x)-gln)
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
	subroutine gcf(gammcf,a,x,gln)
	integer itmax
	real*8 a,gammcf,gln,x,eps,fpmin
	parameter(itmax=100,eps=3.d-7,fpmin=1.d-30)
	integer i
	real*8 an,b,c,d,del,h,gammln
	gln=gammln(a)
	b=x+1.d0-a
	c=1.d0/fpmin
	d=1.d0/b
	h=d
	do i=1,itmax
	an=-i*(i-a)
	b=b+2.d0
	d=an*d+b
	if(dabs(d).lt.fpmin) d=fpmin
	c=b+an/c
	if(dabs(c).lt.fpmin) c=fpmin
	d=1./d
	del=d*c
	h=h*del
	if(dabs(del-1.d0).lt.eps) goto 13
	enddo
	write(6,*) 'a too large, itmax too small in gcf'
	stop
13	gammcf=dexp(-x+a*dlog(x)-gln)*h
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
	FUNCTION ERF(x)
	real*8 erf,x
	real*8 gammp
	if(x.lt.0.d0) then
	erf=-gammp(0.5d0,x**2)
	else
	erf=gammp(0.5d0,x**2)
	endif
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************		
	
	FUNCTION DAWSON(X)
	integer nmax
	real*8 dawson,x,h,a1,a2,a3
	parameter(nmax=6,h=0.4d0,a1=2.d0/3.d0,a2=0.4d0,a3=2.d0/7.d0)
	integer i,init,n0
	real*8 d1,d2,e1,e2,sum,x2,xp,xx,c(nmax)
	save init,c
	data init/0/
	if (init.eq.0) then
		init=1
		do i=1,nmax
		c(i)=dexp(-((2.d0*dfloat(i)-1.d0)*h)**2.d0)
		enddo 
	endif
	if (abs(x).lt.0.2d0) then
	x2=x**2.d0
	dawson=x*(1.d0-a1*x2*(1.d0-a2*x2*(1.d0-a3*x2)))
	else
	xx=abs(x)
	n0=2.d0*nint(0.5d0*xx/h)
	xp=xx-dfloat(n0)*h
	e1=dexp(2.d0*xp*h)
	e2=e1**2.d0
	d1=dfloat(n0+1)
	d2=d1-2.d0
	sum=0.d0
	do i=1,nmax
		sum=sum+c(i)*(e1/d1+1.d0/(d2*e1))
		d1=d1+2.d0
		d2=d2-2.d0
		e1=e2*e1
	enddo 
	dawson=0.5641895835d0*sign(dexp(-xp**2),x)*sum
	endif
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************
		
	function erfcc(x)
	real*8 erfcc,x
	real*8 t,z
	z=dabs(x)
	t=1.d0/(1.d0+0.5d0*z)
	erfcc=t*dexp(-z*z-1.26551223d0+t*(1.00002368d0+t*
     ^	(0.37409196d0+t*(0.09678418d0+t*(-0.18628806d0+t*
     ^	(0.27886807d0+t*(-1.13520398d0+t*(1.48851587d0+t*
     ^	(-0.82215223d0+t*0.17087277)))))))))
     	if (x.eq.0.d0) erfcc=1.d0
     	if (x.lt.0.d0) erfcc=2.d0-erfcc
	return
	end
	
c	****************************************************************
c	****************************************************************
c	****************************************************************	
	
	function funcd(x)
	real*8 x,funcd,erfcc,pi
	external erfcc
	
	pi=4.d0*datan(1.d0)
	
	if (x .ge. 0.d0) then
	if (x .le. 15.d0) then
	funcd=dexp(x)*erfcc(dsqrt(x))
	else
	funcd=(1.d0/dsqrt(pi*x))*(1.d0-(0.5d0/x)+(0.75d0/(x**2))-
     ^	(1.875d0/(x**3)))
      endif
	else
	funcd=dexp(x)
	endif
	
	return
	end	
	
c	****************************************************************
c	****************************************************************
c	****************************************************************
	
	function funcc(x)
	real*8 x,funcc,dawson,pi
	external dawson
	
	pi=4.d0*datan(1.d0)
	
	if (x .ge. 0.d0) then
	if (x .le. 10.d0) then
	funcc=(2.d0/dsqrt(pi))*dawson(dsqrt(x))
	else
	funcc=(1.d0/dsqrt(pi*x))*(1.d0+(0.5d0/x)+(0.75d0/(x**2))+
     ^	(1.875d0/(x**3)))
     	endif
	else
	funcc=0.d0
	endif
	
	return
	end		
		
